/*
 * STRUMPACK -- STRUctured Matrices PACKage, Copyright (c) 2014, The Regents of
 * the University of California, through Lawrence Berkeley National Laboratory
 * (subject to receipt of any required approvals from the U.S. Dept. of Energy).
 * All rights reserved.
 *
 * If you have questions about your rights to use or distribute this software,
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 *
 * NOTICE. This software is owned by the U.S. Department of Energy. As such, the
 * U.S. Government has been granted for itself and others acting on its behalf a
 * paid-up, nonexclusive, irrevocable, worldwide license in the Software to
 * reproduce, prepare derivative works, and perform publicly and display publicly.
 * Beginning five (5) years after the date permission to assert copyright is
 * obtained from the U.S. Department of Energy, and subject to any subsequent five
 * (5) year renewals, the U.S. Government is granted for itself and others acting
 * on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the
 * Software to reproduce, prepare derivative works, distribute copies to the
 * public, perform publicly and display publicly, and to permit others to do so.
 *
 * Developers: Francois-Henry Rouet, Xiaoye S. Li, Pieter Ghysels
 *             (Lawrence Berkeley National Lab, Computational Research Division).
 *
 */

#ifndef MEMORYTRACKER_HPP
#define MEMORYTRACKER_HPP

#if defined(MEMTRACK)
#define DWORD sizeof(double)

class MemoryTracker {
private:
  static double current_size;
  static double max_size;

public:
  MemoryTracker() {
    current_size=0;
    max_size=0;
  }

  static void inc(size_t amount) {
    current_size+=amount;
    max_size=std::max(max_size,current_size);
  }

  static void dec(size_t amount) {
    current_size-=amount;
  }

  static void print() {
    std::cout << "current: " << current_size << "; used: " << max_size << " bytes" << std::endl;
  }

  static double get() {
    return max_size;
  }

  static void reset() {
    current_size=0;
  }
};

double MemoryTracker::current_size = 0;
double MemoryTracker::max_size = 0;

void* operator new(size_t size) throw(std::bad_alloc) {
  char *buf=(char *) malloc(size+DWORD);
  MemoryTracker::inc(static_cast<double>(size));
  ((int *) buf)[0] = size;
  return (void *) (buf+DWORD);
}

void operator delete(void* ptr) throw() {
  char *p=((char *)ptr)-DWORD;
  MemoryTracker::dec(static_cast<double>(((int *) p)[0]));
  free(p);
}

#endif

#endif // MEMORYTRACKER_HPP
