/*
 * STRUMPACK -- STRUctured Matrices PACKage, Copyright (c) 2014, The Regents of
 * the University of California, through Lawrence Berkeley National Laboratory
 * (subject to receipt of any required approvals from the U.S. Dept. of Energy).
 * All rights reserved.
 *
 * If you have questions about your rights to use or distribute this software,
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 *
 * NOTICE. This software is owned by the U.S. Department of Energy. As such, the
 * U.S. Government has been granted for itself and others acting on its behalf a
 * paid-up, nonexclusive, irrevocable, worldwide license in the Software to
 * reproduce, prepare derivative works, and perform publicly and display publicly.
 * Beginning five (5) years after the date permission to assert copyright is
 * obtained from the U.S. Department of Energy, and subject to any subsequent five
 * (5) year renewals, the U.S. Government is granted for itself and others acting
 * on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the
 * Software to reproduce, prepare derivative works, distribute copies to the
 * public, perform publicly and display publicly, and to permit others to do so.
 *
 * Developers: Francois-Henry Rouet, Xiaoye S. Li, Pieter Ghysels
 *             (Lawrence Berkeley National Lab, Computational Research Division).
 *
 */

#ifndef BLAS_H
#define BLAS_H

/* BLAS routines */

  /* xGEMM*/
  void dgemm_(char*, char*, int *, int *, int *,   double *,    double *, int *,  double *, int *,   double *,   double *, int *);
  void sgemm_(char*, char*, int *, int *, int *,   float  *,   float  *, int *,   float  *, int *,   float  *,   float  *, int *);
  void zgemm_(char*, char*, int *, int *, int *, dcomplex *, dcomplex *, int *, dcomplex *, int *, dcomplex *, dcomplex *, int *);
  void cgemm_(char*, char*, int *, int *, int *, scomplex *, scomplex *, int *, scomplex *, int *, scomplex *, scomplex *, int *);

  /* xNRM2 */
  double dnrm2_ (int *,   double *, int *);
   float snrm2_ (int *,    float *, int *);
  double dznrm2_(int *, dcomplex *, int *);
   float scnrm2_(int *, scomplex *, int *);

  /* xSCAL */
  void dscal_(int *,   double *,   double *, int *);
  void sscal_(int *,    float *,    float *, int *);
  void zscal_(int *, dcomplex *, dcomplex *, int *);
  void cscal_(int *, scomplex *, scomplex *, int *);

  /* xGEMV */
  void dgemv_(char*, int*, int*,   double*,   double*, int*,   double*, int*,   double*,   double*, int*);
  void sgemv_(char*, int*, int*,    float*,    float*, int*,    float*, int*,    float*,    float*, int*);
  void zgemv_(char*, int*, int*, dcomplex*, dcomplex*, int*, dcomplex*, int*, dcomplex*, dcomplex*, int*);
  void cgemv_(char*, int*, int*, scomplex*, scomplex*, int*, scomplex*, int*, scomplex*, scomplex*, int*);

  /* xTRSM */
  void dtrsm_(char *, char *, char*, char*, int*, int*,   double*,   double*, int*,   double*, int*);
  void strsm_(char *, char *, char*, char*, int*, int*,    float*,    float*, int*,    float*, int*);
  void ztrsm_(char *, char *, char*, char*, int*, int*, dcomplex*, dcomplex*, int*, dcomplex*, int*);
  void ctrsm_(char *, char *, char*, char*, int*, int*, scomplex*, scomplex*, int*, scomplex*, int*);

  /* xGERC */
  void dger_ (int *m, int *n,   double *alpha,   double *X, int *incx,   double *Y, int *incy,   double *A, int *lda);
  void sger_ (int *m, int *n,    float *alpha,    float *X, int *incx,    float *Y, int *incy,    float *A, int *lda);
  void zgeru_(int *m, int *n, dcomplex *alpha, dcomplex *X, int *incx, dcomplex *Y, int *incy, dcomplex *A, int *lda);
  void cgeru_(int *m, int *n, scomplex *alpha, scomplex *X, int *incx, scomplex *Y, int *incy, scomplex *A, int *lda);

  /* xSWAP */
  void dswap_(int *,   double *, int *,   double *, int *);
  void sswap_(int *,    float *, int *,    float *, int *);
  void zswap_(int *, dcomplex *, int *, dcomplex *, int *);
  void cswap_(int *, scomplex *, int *, scomplex *, int *);

#endif // BLAS_H
